@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2010  Sakari Hyoty
+  Copyright (C) 2010, 2011  Sakari Hyoty
 
   This file is part of Pomodoro.
 
@@ -21,6 +21,7 @@
 #include <QInputDialog>
 #include <QMessageBox>
 #include <phonon/AudioOutput>
+#include <QSettings>
 
 #ifdef Q_WS_MAEMO_5
 #include <QtDBus/QDBusMessage>
@@ -33,6 +34,7 @@
 #include "pomodorotimer.h"
 #include "pomodoroconfig.h"
 #include "taskdialog.h"
+#include "sounddialog.h"
 
 QString secondsToString(int totalSeconds)
 {
@@ -109,6 +111,8 @@ MainWindow::MainWindow(QWidget *parent) :
     // initialize tableView
     QTableView *tableView = ui->tableView;
     tableView->setModel(pomodoroModel);
+    connect(tableView, SIGNAL(doubleClicked(QModelIndex)),
+            this, SLOT(editTask(QModelIndex)));
 
     // initialize headerView
     QHeaderView *headerView = tableView->horizontalHeader();
@@ -124,12 +128,27 @@ MainWindow::MainWindow(QWidget *parent) :
             this, SLOT(taskSelected(QItemSelection, QItemSelection)));
 
     taskDialog = new TaskDialog(this);
-    connect(ui->addButton, SIGNAL(clicked()), taskDialog, SLOT(show()));
+    connect(ui->addButton, SIGNAL(clicked()), taskDialog, SLOT(showAdd()));
     connect(taskDialog, SIGNAL(newTask(QString,int)),
             this, SLOT(addTask(QString,int)));
+    connect(taskDialog, SIGNAL(taskUpdated(QModelIndex, QString, int)),
+            this, SLOT(taskUpdated(QModelIndex, QString, int)));
+
+    soundDialog = new SoundDialog(this);
+    connect(soundDialog, SIGNAL(alarmFileChanged(QString)),
+            this, SLOT(alarmFileChanged(QString)));
+    connect(soundDialog, SIGNAL(alarmFileReset()),
+            this, SLOT(alarmFileReset()));
 
     connect(ui->actionGuide, SIGNAL(triggered()), this, SLOT(showGuide()));
     connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
+    connect(ui->actionRemoveCompleted, SIGNAL(triggered()), this, SLOT(removeCompletedTasks()));
+    connect(ui->actionChangeAlarm, SIGNAL(triggered()), this, SLOT(showChangeAlarmFile()));
+
+    // load alarm sound file path from settings
+    QSettings settings(POMODORO_FILE_SETTINGS, QSettings::IniFormat);
+    alarmFile = settings.value(POMODORO_SETTINGS_KEY_ALARM,
+                               POMODORO_FILE_ALERT).toString();
 
     alertMediaObject = new Phonon::MediaObject(this);
     Phonon::AudioOutput *audioOutput =
@@ -168,6 +187,34 @@ void MainWindow::addTask(QString taskName, int estimate)
     }
 }
 
+void MainWindow::editTask(const QModelIndex &index)
+{
+    qDebug("MainWindow::editTask");
+
+    if (!pomodoroModel->isTaskCompleted(index)) {
+        taskDialog->showEdit(index, pomodoroModel->taskName(index),
+                             pomodoroModel->estimate(index));
+    } else {
+        showSystemNotification(tr("Unable to edit a completed task"));
+    }
+}
+
+void MainWindow::taskUpdated(const QModelIndex &index,
+                             QString taskName, int estimate)
+{
+    qDebug("MainWindow::taskUpdated");
+
+    selectedIndex = index;
+
+    pomodoroModel->updateTask(selectedIndex, taskName, estimate);
+
+    setUiState(IdleTaskSelected);
+    ui->tableView->setCurrentIndex(selectedIndex);
+    ui->tableView->scrollTo(
+        selectedIndex,
+        QAbstractItemView::PositionAtCenter);
+}
+
 void MainWindow::removeTask()
 {
     qDebug("MainWindow::removeTask");
@@ -251,8 +298,15 @@ void MainWindow::pomodoroStopped(bool finished)
 
     if (finished) {
         pomodoroModel->pomodoroCompleted(selectedIndex);
-        showSystemNotification(tr("Pomodoro finished"));
-        alertMediaObject->setCurrentSource(POMODORO_FILE_ALERT);
+
+        if (QFile(alarmFile).exists()) {
+            showSystemNotification(tr("Pomodoro finished"));
+            alertMediaObject->setCurrentSource(alarmFile);
+        } else {
+            showSystemNotification(tr("Selected custom alarm file does not exist"));
+            alertMediaObject->setCurrentSource(POMODORO_FILE_ALERT);
+        }
+
         alertMediaObject->play();
     }
 
@@ -289,13 +343,14 @@ void MainWindow::showGuide()
     QMessageBox::information(this, title, text);
 
     title = tr("How to use this application?");
-    text = tr("First add a task and estimate it in pomodoros. "
-              "Then just start the task and work until "
-              "time runs out. For every interruption during a pomodoro click "
+    text = tr("Add a task and estimate it in pomodoros. "
+              "Start the task and work until the "
+              "time runs out. Interruptions can be recorded by clicking "
               "on Int./Ext. button depending on the distraction type."
               "\n\n"
-              "Mark the task completed when it's done. Completed tasks will "
-              "be hidden from the task list the next day.\n\n"
+              "Mark the task completed when it's done. Completed tasks can "
+              "be mass removed from the menu. Tasks can be edited "
+              "by double clicking.\n\n"
               "You can access this guide again at any time from the application menu.");
 
     QMessageBox::information(this, title, text);
@@ -307,15 +362,52 @@ void MainWindow::showAbout()
 
     QString title = tr("About");
     QString text =
-            tr("Pomodoro  Copyright (C) 2010 Sakari Hyoty\n\n"
-               "%1\n\n"
+            tr("Pomodoro %1\nCopyright (C) 2010, 2011 Sakari Hyoty\n\n"
+               "%2\n\n"
                "This program comes with ABSOLUTELY NO WARRANTY. This is free software, "
-               "and you are welcome to redistribute it under certain conditions; see `%2' "
-               "for details.").arg(POMODORO_HOMEPAGE).arg(POMODORO_README_FILE);
+               "and you are welcome to redistribute it under certain conditions; see `%3' "
+               "for details.").arg(POMODORO_VERSION).arg(POMODORO_HOMEPAGE).arg(POMODORO_README_FILE);
 
     QMessageBox::about(this, title, text);
 }
 
+void MainWindow::removeCompletedTasks()
+{
+    qDebug("MainWindow::removeCompletedTasks");
+
+    pomodoroModel->deleteCompletedTasks();
+    selectedIndex = QModelIndex();
+    setUiState(IdleNoSelection);
+}
+
+void MainWindow::showChangeAlarmFile()
+{
+    qDebug("MainWindow::showChangeAlarmFile");
+
+    soundDialog->show(alarmFile);
+}
+
+void MainWindow::alarmFileChanged(QString newAlarmFile)
+{
+    qDebug("MainWindow::alarmFileChanged");
+
+    alarmFile = newAlarmFile;
+
+    QSettings settings(POMODORO_FILE_SETTINGS, QSettings::IniFormat);
+    settings.setValue(POMODORO_SETTINGS_KEY_ALARM, alarmFile);
+    settings.sync();
+}
+
+void MainWindow::alarmFileReset()
+{
+    qDebug("MainWindow::alarmFileReset");
+
+    alarmFile = POMODORO_FILE_ALERT;
+
+    QFile file(POMODORO_FILE_SETTINGS);
+    file.remove();
+}
+
 void MainWindow::changeEvent(QEvent *e)
 {
     //qDebug("MainWindow::changeEvent");
@@ -338,6 +430,8 @@ void MainWindow::setUiState(PomodoroUiState state)
     // developer: keep lines alphabetically sorted
     switch (state) {
     case IdleNoSelection:
+        ui->actionChangeAlarm->setEnabled(true);
+        ui->actionRemoveCompleted->setEnabled(true);
         ui->currentTaskName->setText(QString(""));
         ui->currentTaskName->setVisible(false);
         ui->doneButton->setEnabled(false);
@@ -350,6 +444,8 @@ void MainWindow::setUiState(PomodoroUiState state)
         ui->tableView->setVisible(true);
         break;
     case IdleTaskSelected:
+        ui->actionChangeAlarm->setEnabled(true);
+        ui->actionRemoveCompleted->setEnabled(true);
         ui->currentTaskName->setText(QString(""));
         ui->currentTaskName->setVisible(false);
         ui->doneButton->setEnabled(true);
@@ -362,6 +458,8 @@ void MainWindow::setUiState(PomodoroUiState state)
         ui->tableView->setVisible(true);
         break;
     case IdleCompletedTaskSelected:
+        ui->actionChangeAlarm->setEnabled(true);
+        ui->actionRemoveCompleted->setEnabled(true);
         ui->currentTaskName->setText(QString(""));
         ui->currentTaskName->setVisible(false);
         ui->doneButton->setEnabled(false);
@@ -374,6 +472,8 @@ void MainWindow::setUiState(PomodoroUiState state)
         ui->tableView->setVisible(true);
         break;
     case Started:
+        ui->actionChangeAlarm->setEnabled(false);
+        ui->actionRemoveCompleted->setEnabled(false);
         ui->currentTaskName->setVisible(true);
         ui->doneButton->setEnabled(false);
         ui->externalButton->setEnabled(true);
@@ -385,6 +485,8 @@ void MainWindow::setUiState(PomodoroUiState state)
         ui->tableView->setVisible(false);
         break;
     case AlmostFinished:
+        ui->actionChangeAlarm->setEnabled(false);
+        ui->actionRemoveCompleted->setEnabled(false);
         ui->currentTaskName->setVisible(true);
         ui->doneButton->setEnabled(true);
         ui->externalButton->setEnabled(true);
