@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2010  Sakari Hyoty
+  Copyright (C) 2010, 2011  Sakari Hyoty
 
   This file is part of Pomodoro.
 
@@ -38,11 +38,6 @@ struct PomodoroTask
     {
     }
 
-    bool displayInTaskList() const
-    {
-        return !isCompleted() || completionDate == QDate::currentDate();
-    }
-
     bool isCompleted() const
     {
         return completionDate.isValid();
@@ -84,9 +79,7 @@ bool modelLoader_1(QDataStream &readStream, QVector<PomodoroTask> &vector)
                 >> task.creationDate
                 >> task.completionDate;
 
-        if (task.displayInTaskList()) {
-            vector.append(task);
-        }
+        vector.append(task);
     }
 
     return readStream.status() == QDataStream::Ok;
@@ -96,7 +89,7 @@ bool modelSaver_1(QDataStream &writeStream, const QVector<PomodoroTask> &vector)
 {
     PomodoroTask task;
 
-    for (int i = 0; i < vector.count(); i++) {
+    for (int i = 0; i < vector.count() && writeStream.status() == QDataStream::Ok; i++) {
         task = vector.at(i);
         writeStream
                 << task.id
@@ -109,7 +102,7 @@ bool modelSaver_1(QDataStream &writeStream, const QVector<PomodoroTask> &vector)
                 << task.completionDate;
     }
 
-    return true;
+    return writeStream.status() == QDataStream::Ok;
 }
 
 PomodoroModel::PomodoroModel(QObject *parent) :
@@ -270,15 +263,15 @@ bool PomodoroModel::saveModel()
             QDataStream readWriteStream(&file);
             readWriteStream.setVersion(QDataStream::Qt_4_5);
 
-            int version = 0;
+            int version = 0; // default to not-a-version
 
             readWriteStream >> version;
 
-            if (!exists || version == 1) {
+            if (version == 1 || !exists) {
                 file.seek(0);
-                file.resize(0);
+                file.resize(0); // reset file size to zero
                 readWriteStream.setDevice(&file);
-                readWriteStream << 1;
+                readWriteStream << 1; // data format version
                 readWriteStream << taskIdRunner;
                 retval = modelSaver_1(readWriteStream, pomodoroVector);
                 dirty = false;
@@ -313,6 +306,16 @@ QModelIndex PomodoroModel::newTask(QString taskName, int estimate)
     return createIndex(lastRow, 0, pomodoroVector.at(lastRow).id);
 }
 
+void PomodoroModel::updateTask(const QModelIndex &index, QString taskName, int estimate)
+{
+    PomodoroTask *t = task(index);
+    t->taskName = taskName;
+    t->pomodorosEstimated = estimate;
+    dirty = true;
+    saveModel();
+    reset(); // notify view
+}
+
 void PomodoroModel::deleteTask(const QModelIndex &index)
 {
     qDebug("PomodoroModel::deleteTask");
@@ -364,7 +367,33 @@ void PomodoroModel::pomodoroInterrupted(const QModelIndex &index, bool internalI
     reset(); // notify view
 }
 
-PomodoroTask *PomodoroModel::task(const QModelIndex &index)
+void PomodoroModel::deleteCompletedTasks()
+{
+    qDebug("PomodoroModel::deleteCompletedTasks");
+
+    for (int i = 0; i < pomodoroVector.count();) {
+        if (pomodoroVector[i].isCompleted())
+            pomodoroVector.remove(i);
+        else
+            i++;
+    }
+
+    dirty = true;
+    saveModel();
+    reset(); // notify view
+}
+
+QString PomodoroModel::taskName(const QModelIndex &index)
+{
+    return task(index)->taskName;
+}
+
+int PomodoroModel::estimate(const QModelIndex &index)
+{
+    return task(index)->pomodorosEstimated;
+}
+
+PomodoroTask * PomodoroModel::task(const QModelIndex &index)
 {
     qDebug("PomodoroModel::task");
 
