@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2010  Sakari Hyoty
+  Copyright (C) 2010, 2011  Sakari Hyoty
 
   This file is part of Pomodoro.
 
@@ -22,6 +22,8 @@
 
 #include <QDialog>
 
+class QModelIndex;
+
 namespace Ui {
     class TaskDialog;
 }
@@ -33,19 +35,23 @@ public:
     ~TaskDialog();
 
 public slots:
-    void show(); // reimplemented from QDialog
+    void showAdd();
+    void showEdit(const QModelIndex& index, QString taskName, int estimate);
     void accept(); // reimplemented from QDialog
     void reject(); // reimplemented from QDialog
     void taskNameChanged(QString newName);
 
 signals:
     void newTask(QString taskName, int estimate);
+    void taskUpdated(const QModelIndex& index, QString taskName, int estimate);
 
 protected:
     void changeEvent(QEvent *e);
 
 private:
     Ui::TaskDialog *ui;
+
+    const QModelIndex *modelIndex;
 };
 
 #endif // TASKDIALOG_H
